#define CPUTEMP_FILE    "/sys/class/thermal/thermal_zone0/temp"

#include <cstdint>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <chrono>
#include <pthread.h>
#include <unistd.h>

using namespace std;

typedef struct _str_val {
    pthread_t th;
    pthread_mutex_t mutex;
    volatile int64_t ctr;
} STR_VAL;

void *inc_func(void *arg)
{
    STR_VAL *p  = (STR_VAL *)arg;
    while (1) {
        pthread_mutex_lock(&(p->mutex));
        (p->ctr)++;
        pthread_mutex_unlock(&(p->mutex));
    }
}

int main(int argc, char *argv[])
{
    if (argc != 3) {
        fprintf(stderr, "Usage: %s <thread_count> <loop_count>\n", argv[0]);
        return 1;
    }
    int thread_count  = atoi(argv[1]);
    int loop_count = atoi(argv[2]);

    STR_VAL *str = (STR_VAL *)malloc(sizeof (STR_VAL) * (thread_count + 1));

    // スレッドを起動
    for (int i = 0; i < thread_count; i++) {
        (str[i]).ctr = 0;
        pthread_mutex_init(&((str[i]).mutex), NULL);
        pthread_create(&((str[i]).th), NULL, inc_func, (void *)(&(str[i])));
    }

    // ミリ秒時刻を取得
    auto start_time = chrono::high_resolution_clock::now();
    auto previous_time = start_time;

    for (int c = 0; c < loop_count; c++) {
        // 1秒待つ
        sleep(1);

        // 前回からの経過時間を求める(ミリ秒)。
        auto current_time = chrono::high_resolution_clock::now();
        double duration = chrono::duration_cast<chrono::milliseconds>(current_time - previous_time).count();
        double elapsed_time = chrono::duration_cast<chrono::milliseconds>(current_time - start_time).count();
        previous_time = current_time;

        // 各スレッドのカウント数の合計を求める。
        int64_t sum = 0;
        for (int i = 0; i < thread_count; i++) {
            pthread_mutex_lock(&((str[i]).mutex));
            sum += (str[i]).ctr;
            (str[i]).ctr = 0;
            pthread_mutex_unlock(&((str[i]).mutex));
        }
        int64_t count_per_second = sum * 1000 / duration;

        // CPU温度を取得
        double temperature = 0;
        try {
            ifstream fptemp(CPUTEMP_FILE);
            int t;
            fptemp >> t;
            fptemp.close();
            temperature = (double)t / 1000;
        } catch (const exception &e) {}

        // 動作クロック周波数を取得
        FILE *fp;
        unsigned int clock_frequency = 0;
        fp = popen("/usr/bin/vcgencmd measure_clock arm | /usr/bin/cut -d '=' -f 2,2", "r");
        if (fp != NULL) {
            fscanf(fp, "%d", &clock_frequency);
            pclose(fp);
        }

        // JSONデータを作成
        cout << "{"
             << "\"elapsed_time\": " << elapsed_time / 1000 << ", "
             << "\"temperature\": " << temperature << ", "
             << "\"clock_frequency\": " << clock_frequency << ", "
             << "\"count_per_second\": " << count_per_second
             << "}"
             << endl;
    }
    return 0;
}
